require(dplyr)
require(stringr)

folder <- choose.dir(caption = "Select folder with 'MasterTable' and other CSV files to load")
setwd(folder)

# load MasterTable. Change for analysis of different data set
MasterTable <- read.csv2(file="MasterTable.csv", header=TRUE, sep=";",
                         na.strings="NA")

#load Table of Conditions. Change for analysis of different data set
Conditions <- read.csv2(file="Conditions.csv", header=TRUE, sep=";")

# only get the unique days and paths for each day
daypathlist <- MasterTable

#determine the number of days
ndays <- length(daypathlist[,1])
alldays <- c() # alldays is the sink for all daily data. reset in case it already exists

#main loop over the experimental days listed in the MasterTable; data from all days is aggregated
if (ndays>0) {
  for (i in 1:ndays) {
    
    nthday <- read.csv2(file=as.character(daypathlist$path[i]),  sep=";", dec = ",", header=TRUE,
                        fileEncoding="UTF-16LE", as.is = T, row.names=NULL) #read csv file with raw data

    nthday <- nthday %>% arrange(DateTime) %>% filter(str_detect(Animal, "B")) #sort chronologically first
    
    #make a day column with the day number 
    nthday$day <- daypathlist[i,1]
    
    #aggregate data from all days in one table
    if (i>1) {
      alldays <- rbind(alldays, nthday)
    } else{
      alldays <- nthday
    }
  }
  
  #reformat DateTime and display properly, as characters
  alldays$DateTime <- sub(",",".",alldays$DateTime)
  alldays$DateTime <- as.character(as.POSIXct(as.numeric(alldays$DateTime) * (60*60*24)
                                              , origin="1899-12-30", tz="GMT"))
  
  #remove columns with irrelevant data
  alldays <- alldays %>% select(day, DateTime, Animal, Unit, EventDuration, Sense1duration,
                                Reinforce1value, Reinforce2value) %>%
    #also remove unnaturally long visits, over 10 seconds long (caused by bugs)
    filter(Sense1duration < 10000)
  
  # make new columns 
  alldays <- alldays %>% filter(Sense1duration < 10000) %>% mutate(
    Reinforce1value = ifelse(is.na(Reinforce1value),0,Reinforce1value),
    Reinforce2value = ifelse(is.na(Reinforce2value),0,Reinforce2value),
    #numeric column for reward volume delivered
    vol = (Reinforce1value + Reinforce2value)*2,
    # numeric column for reward status 1=rewarded, 0=unrewarded, helps calculate reward rates
    rewardstatus = ifelse(vol>0,1,vol),
    #create location column from the unitLabels
    loc = as.integer(str_extract(Unit, "[0-9]+"))) %>%
    select(-Reinforce1value, -Reinforce2value, -Unit)
  
  # merge the Conditions table and the current data table to make new columns for dispenser properties
  # such as maximal volume, sugar concentration, probability, etc., as well as experimental conditions
  # days to discard, etc.
 
  alldays <-merge(alldays, Conditions, 
                  by = c("day", "loc"), all.x = TRUE, sort = FALSE) 
  # sort chronologically again
  alldays <- alldays %>% arrange(DateTime)

  # write output to a csv file
  write.table(alldays, file = "main_data.csv", sep=";", row.names=FALSE)
 
}